// Bs1ueb8.c -- Copyright 2005 by Matthias Jauernig
// Hauptprogramm mit Parentprozess
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdbool.h>

bool ready1;
bool ready2;

void sigusr1fkt(int signr){
	printf("Parent -> Child1 bereit...\n");
	ready1 = true;
}

void sigusr2fkt(int signr){
	printf("Parent -> Child2 bereit...\n");
	ready2 = true;
}

int main(int argc, char **argv){
	int status1, status2, id1, id2, count;
	pid_t pid1, pid2;
	char *c1argv[3], *c2argv[3], *child1="child1", *child2="child2";
	ready1=false;
	ready2=false;
	
	// Argumentenlisten fr Childs fllen
	c1argv[0]=child1;
	c2argv[0]=child2;
	c1argv[1] = (char*)malloc(100);
	if(argc>1){
		count=atoi(argv[1]);
		if(count==0){
			printf("Parent -> Nicht-Zahl uebergeben...\n");
			exit(1);
		}
	}else{
		printf("Parent -> Zaehlzyklenzahl eingeben: ");
		scanf("%d",&count);
	}
	sprintf(c1argv[1],"%d",count);
	// c2argv[1] wird nach Child1-Erzeugung belegt
	c1argv[2]=NULL;
	c2argv[2]=NULL;
	
	// unter Solaris:
	sigset(SIGUSR1, sigusr1fkt);
	sigset(SIGUSR2, sigusr2fkt);
	
	sighold(SIGUSR1);
	sighold(SIGUSR2);
	
	printf("Parent  -> Meine PID:  %d\n",(int)getpid());
	// ersten Child erzeugen
	pid1 = fork();
	c2argv[1] = (char*)malloc(100);
	sprintf(c2argv[1],"%d",(int)pid1);
	
	switch(pid1){
		case -1:
			perror("Parent -> fork pid1");
			exit(1);
			break;
		case 0:
			// Child1 ausfhren
			execv(child1,c1argv);
			perror("Parent -> Fehler bei Ausfhrung von Child1 in Parent");
			break;
		default:
			pid2 = fork();
			switch(pid2){
				case -1:
					perror("Parent -> fork pid2");
					exit(1);
					break;
				case 0:
					// Child2 ausfhren
					execv(child2,c2argv);
					perror("Parent -> Fehler bei Ausfhrung von Child2 in Parent");
					break;
				default:
					// 1.) Auf Signal der beiden Childs warten -> Bereitschaftsmeldung
					sigrelse(SIGUSR1);
					while(!ready1);
					sigrelse(SIGUSR2);
					while(!ready2);
					
					// 2.) Bereitschaftsmeldung an 2. Child schicken
					if(kill(pid2,SIGUSR1) < 0)
						perror("Parent -> Signal senden an Child2");
					
					// 3.) Auf Beendigung der Child-Prozesse warten und deren Ende-Statuswerte ausgeben
					id1=0; id2=0;
					do{
						if(id1==0){
							id1=waitpid(pid1,&status1,WNOHANG);
							if(id1!=0)
								printf("Parent -> Child1 beendet: PID=%d, Status: |%x|%x|\n",
									(int)pid1,(status1>>8)&0xFF,status1&0x7F);
						}
						if(id2==0){
							id2=waitpid(pid2,&status2,WNOHANG);
							if(id2!=0)
								printf("Parent -> Child2 beendet: PID=%d, Status: |%d|%x|\n",
									(int)pid2,(status2>>8)&0xFF,status2&0x7F);
						}
					}while(!id1 || !id2);
					break;
			}
			break;
	}
	
	exit(1);
}
